/***************************************************************************************************
 Project : HW watchdog for Raspberry PI
 Date    : 5/2015
 Author  : Jan Matuska, matuska@starhill.org                          
 Company : www.starhill.org
 Desc.   : True hardware watchdog for Raspberry PI 
 URL     : http://walda.starhill.org/elektro/rpiwdt/
 ---------------------------------------------------------------------------------------------------
 Chip type                : ATTiny85
 Clock frequency          : 1.0 MHz (internal RC)
 Compiler                 : avr-gcc 
 ---------------------------------------------------------------------------------------------------               
***************************************************************************************************/
#define F_CPU 1000000UL  // 1 MHz
#include <avr/io.h>
#include <avr/wdt.h>
#include <util/delay.h>

// RPi reset
#define RPIRST_LOW     DDRB |= (1<<DDB3);       // low state
#define RPIRST_HIGH    DDRB &= ~(1<<DDB3);      // Hi-Z state

// Feedback to RPi
#define FEEDBACK_LOW   DDRB |= (1<<DDB0);     // low state
#define FEEDBACK_HIGH  DDRB &= ~(1<<DDB0);    // Hi-Z state
       
// Clear counter pin
#define CCIN           PINB&(1<<PB4)

// Timeouts in 10ms steps
#define LONGTIME      30000    // long timeour 
#define SHORTTIME      7000    // short timeout
#define FASTCCINTIME    200    // max time to detect fast CCIN impusles 

// Various
#define NUMCCFAST         5   // Amount of fast CCIN impules to set long time 
                                                      
                                                              
// The story begins here
int main(void) {

  unsigned char lastCCINState = 0;
  unsigned char currentCCINState = 0;
  unsigned int counter = 0;
  unsigned int topcnt = LONGTIME;
  unsigned char fastCCINCounter = 0;

  // all as input
  PORTB = 0;
  DDRB = 0;
  // set pullup at CCIN input
  PORTB = (1<<PB4);

  // wdt enable  
  wdt_enable(WDTO_8S);	
  
  // delay after startup
  //_delay_ms(5000);

  // mainloop  
  while(1) {
  
    // increment counter
    counter++;
    
    // read current CCIN state
    currentCCINState = CCIN;
  
    // rising edge of CCIN signal
    if(currentCCINState && !lastCCINState) { 
      if(counter<=FASTCCINTIME) {
        if(fastCCINCounter++>=NUMCCFAST) {
          topcnt = LONGTIME;
          fastCCINCounter = 0;
        }
      } else {
        fastCCINCounter = 0;
        topcnt = SHORTTIME;
      }      
      counter = 0;
    }
    
    // feedback
    if(currentCCINState) {
      FEEDBACK_HIGH;
    } else {
      FEEDBACK_LOW;
    }
  
    // make raspberry pi reset
    if(counter == topcnt) {
      RPIRST_LOW;
      _delay_ms(2000);
      RPIRST_HIGH;
      counter = 0;
      topcnt = LONGTIME; 
    }
  
    // store current CCIN state
    lastCCINState = currentCCINState;
    
    // internal watchdog reset
    wdt_reset();
    
    // loop delay
    _delay_ms(10);

  }

}
